Type TTranslator
	Global tlLanguages:TList
	Global tmSavedStrings:TMap
	Global tmKeyNames:TMap
	Global tmPathesToLoad:TMap
	Global sCurrentLanguage:String
	
	rem
		about: This function initializes the Translator. It reads out all the files. Required to call before using the Translator.
		@psPathesPath has to be the path to the file where the alternative pathes for images sounds etc. which depend on the language are.
		@psStringPath has to be the path to the file where the strings (ingame texts) are.
		@psKeyPath has to be the path to the file where the keynames are.
		@psAvailableLanguagesPath has to be the path to the file where the available languages are.
	endrem
	Function Initialize(psPathesPath:String, psStringsPath:String, psKeyPath:String, psAvailableLanguagesPath:String)
		tlLanguages = New TList
		tmSavedStrings = New TMap
		tmKeyNames = New TMap
		tmPathesToLoad = New TMap
		
		LoadLanguages(psAvailableLanguagesPath)
		LoadKeyNames(psKeyPath)
		LoadStrings(psStringsPath)
		LoadPathes(psPathesPath)
		
		sCurrentLanguage = Lower(String(tlLanguages.Last()))
		
	End Function
	
	rem
		about: This function reads all available languages from the file @psAvailableLanguagesPath.
	endrem
	Function LoadLanguages(psAvailableLanguagesPath:String)
		Local tsStream:TStream = ReadFile(psAvailableLanguagesPath)
		
		Repeat
			Local sLine:String = tsStream.ReadLine()
			tlLanguages.AddLast(sLine)
		Until(tsStream.Eof())
		
		tsStream.Close()
	End Function
	
	rem
		about: This function reads all keynames in all languages from the file @psKeyPath.
	endrem
	Function LoadKeyNames(psKeyPath:String)
		Local tsStream:TStream
		
		For Local lang:String = EachIn tlLanguages
			tsStream = ReadFile(psKeyPath)
			__ReadKeysForLanguage(lang, tsStream)
			CloseFile(tsStream)
		Next

	End Function
	
	rem
		about: This function reads all strings in all languages from the file @psStringPath.
	endrem
	Function LoadStrings(psStringsPath:String)
		Local tsStream:TStream
		
		For Local lang:String = EachIn tlLanguages
			tsStream = ReadFile(psStringsPath)
			__ReadStringsForLanguage(lang, tsStream)
			CloseFile(tsStream)
		Next

	End Function
	
	rem
		about: This function reads all pathes (for alternative files) from the file @psPathesPath.
	endrem
	Function LoadPathes(psPathesPath:String)
		Local tsStream:TStream
		
		For Local Lang:String = EachIn tlLanguages
			tsStream = ReadFile(psPathesPath)
			__ReadPathesForLanguage(Lang, tsStream)
			CloseFile(tsStream)
		Next
	End Function
	
	rem
		about: This function reads all strings for a given language @psLanguage from the file @ptsStream.
	endrem
	Function __ReadStringsForLanguage(psLanguage:String, ptsStream:TStream)
		Local sLang:String = "[" + psLanguage + "]"
		
		While Not (Eof(ptsStream))
			Local sLine:String = ptsStream.ReadLine()
			If(Lower(sLine) = Lower(sLang)) Then
				Local bEnd:Byte = False
				Repeat
					bEnd = False
					sLine = ptsStream.ReadLine()
					bEnd = (Left(sLine, 1) = "[")
					bEnd = bEnd Or (Trim(sLine) = "")
					If(Not bEnd) Then
						Local sStringName:String = GetLeftPart(sline)
						Local sStringValue:String = GetValue(sLine)
						tmSavedStrings.Insert(Lower(sStringName) + Lower(psLanguage), sStringValue)
					EndIf
				Until bEnd
			EndIf
		Wend
	End Function
	
	rem
		about: This function reads all keys for a given language @psLanguage from the file @ptsStream. 
	endrem
	Function __ReadKeysForLanguage(psLanguage:String, ptsStream:TStream)
		Local sLang:String = "[" + psLanguage + "]"
		
		While Not(Eof(ptsStream))
			Local sLine:String = ptsStream.ReadLine()
			If(Lower(sLine) = Lower(sLang)) Then
				Repeat
					sLine = ptsStream.ReadLine()
					If(Not(Left(sLine, 1) = "[")) Then
						If(Not (Trim(sLine) = "")) Then
							Local sKeyName:String = GetLeftPart(sLine)
							Local sKeyNumber:String = GetValue(sline)
							tmKeyNames.Insert(sKeyNumber + Lower(psLanguage), sKeyName)
						EndIf
					EndIf
				Until(Left(sLine, 1) = "[" Or Eof(ptsStream))
			EndIf
		WEnd
	End Function
	
	rem
		about: This function reads all paths for a given language @psLanguage from the file @ptsStream.
	endrem
	Function __ReadPathesForLanguage(psLanguage:String, ptsStream:TStream)
		Local sLang:String = "[" + psLanguage + "]"
		
		While Not(Eof(ptsStream))
			Local sLine:String = ptsStream.ReadLine()
			If(Lower(sLine) = Lower(sLang)) Then
				Local bEnd:Byte = False
				Repeat
					bEnd = False
					sLine = ptsStream.ReadLine()
					bEnd = (Left(sLine, 1) = "[")
					bEnd = bEnd Or (Trim(sLine) = "")
					If(Not bEnd) Then
						Local sDefaultPath:String = GetLeftPart(sline)
						Local sLanguagePath:String = GetValue(sLine)
						DebugLog("Inserted: " + sLanguagePath + " with: " + sDefaultPath + Lower(psLanguage))
						tmPathesToLoad.Insert(sDefaultPath + Lower(psLanguage), sLanguagePath)
					EndIf
				Until bEnd
			EndIf
		Wend
	End Function
	
	rem
		about: This function returns a file path of a file which depends on the language. Therefore it needs the default path @psDefaultPath and the current language @psLanguage.
		If the file is not available in the chosen language or it doesn't depend on the chosen language, it just returns the default path.
	endrem
	Function GetFilePath:String(psDefaultPath:String, psLanguage:String = "")
		If(psLanguage = "") Then psLanguage = sCurrentLanguage
		DebugLog("Debug: " + psDefaultPath)
		
		If Not(tmPathesToLoad.Contains(psDefaultPath + Lower(psLanguage))) Then
			DebugLog("Returns: " + psDefaultPath)
			Return psDefaultPath
		EndIf
		
		DebugLog("Should be: " + String(tmPathesToLoad.ValueForKey(psDefaultPath + Lower(psLanguage))))
		
		Return String(tmPathesToLoad.ValueForKey(psDefaultPath + Lower(psLanguage)))
	End Function
	
	rem
		about: This function returns the name of the key with the number @piKeyNumber in the language @psLanguage.
	endrem
	Function GetKeyName:String(piKeyNumber:Int, psLanguage:String = "")
		If(psLanguage = "") Then psLanguage = sCurrentLanguage
		
		Return String(tmKeyNames.ValueForKey(String(piKeyNumber) + Lower(psLanguage)))
	End Function
	
	rem
		about: This function returns the string which belongs to the name @psStringName in the language @psLanguage without parsing any variables.
	endrem
	Function GetSingleString:String(psStringName:String, psLanguage:String = "")
		If(psLanguage = "") Then psLanguage = sCurrentLanguage
		
		Return String(tmSavedStrings.ValueForKey(Lower(psStringName) + Lower(psLanguage)))
	End Function
	
	rem
		about: This function returns a string which belogs to the name @psStringName in the language @psLanguage by replacing the variables with strings from the array @psVariables.
	endrem
	Function GetFixedString:String(psStringName:String, psVariables:String[], psLanguage:String = "")
		If(psLanguage = "") Then psLanguage = sCurrentLanguage
			
		Return __FixString(GetSingleString(psStringName, psLanguage), psVariables)
	End Function
	
	rem
		about: This function replaces all variables in the string @psString with values from @psVariables.
	endrem
	Function __FixString:String(psString:String, psVariables:String[])
		For Local index:Int = 0 Until psVariables.Length
			If (psString.Contains("&" + String(index + 1))) Then
				psString = psString.Replace("&" + String(index + 1), psVariables[index])
			EndIf
		Next
		
		Return psString
	End Function
	
	rem
		about: Returns the value of a line @psLine from an Ini-File. (It returns what is on the right side of "=")
	endrem
	Function GetValue:String(psLine:String)
		If Instr(psLine, "=") > 0 Then
			Return Right(psLine, Len(psLine) - Instr(psLine, "="))
		Else
			Return ""
		EndIf
	End Function

	rem
		about: Returns the keyword of a line @psLine from an Ini-File. (It returns what is on the left side of "=")
	endrem
	Function GetLeftPart:String(psLine:String)
		If Instr(psLine, "=") > 0 Then
			Return Left(psLine, Instr(psLine, "=") - 1)
		Else
			Return ""
		EndIf
	End Function
End Type